/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2004
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 * 
 * $Id: CoCompose.java,v 1.2 2004/05/07 14:19:28 dwagelaa Exp $
 * $Log: CoCompose.java,v $
 * Revision 1.2  2004/05/07 14:19:28  dwagelaa
 * + added model pool (shared model) support
 * + added inter-model reference support
 *
 */

package be.ac.vub.cocompose;

import java.util.Collection;
import java.util.Hashtable;
import java.util.List;
import java.util.Properties;

import be.ac.vub.cocompose.lang.DefaultTraversingModelVisitor;
import be.ac.vub.cocompose.lang.ModelElementException;
import be.ac.vub.cocompose.lang.ModelFactory;
import be.ac.vub.cocompose.lang.core.Model;
import be.ac.vub.cocompose.lang.core.ModelElement;
import be.ac.vub.cocompose.lang.core.RefinedElement;
import be.ac.vub.cocompose.log.DefaultLog;
import be.ac.vub.cocompose.log.Log;

/**
 * Main CoCompose class for stand-alone use and settings registry.
 * @author Dennis Wagelaar
 * @version 0.2.10, 06/05/2004
 * @since 0.2.8
 */
public class CoCompose
{
  /**
   * Removes all shared models referenced from the given model.
   * @since 0.2.10
   */
  class RemoveReferencedModels extends DefaultTraversingModelVisitor
  {
    /**
     * @see be.ac.vub.cocompose.lang.ModelVisitor#visitRefinedElement(be.ac.vub.cocompose.lang.core.RefinedElement)
     */
    public void visitRefinedElement(RefinedElement refinedElement) throws ModelElementException
    {
      super.visitRefinedElement(refinedElement);
      Model model = getModel(refinedElement);
      List inh = refinedElement.getInheritsFrom();
      for (int i = 0; i < inh.size(); i++)
      {
        RefinedElement elt = (RefinedElement) inh.get(i);
        Model emodel = getModel(elt);
        if (!model.equals(emodel))
        {
          removeSharedModel(emodel);
        }
      }
      List sup = refinedElement.getSuperimposesOn();
      for (int i = 0; i < sup.size(); i++)
      {
        RefinedElement elt = (RefinedElement) sup.get(i);
        Model emodel = getModel(elt);
        if (!model.equals(emodel))
        {
          removeSharedModel(emodel);
        }
      }
    }
  
    /**
     * Gets the model containing the given element. 
     * @param element the element for which to get the model.
     * @return the model containing the given element.
     * @throws ModelElementException if the element has no model.
     */
    private Model getModel(ModelElement element) throws ModelElementException
    {
      Model model = element.getModel();
      if (model == null)
      {
        throw new ModelElementException(
          "Model must be set for each element",
          element);
      }
      return model;
    }
  }
  
  private static Properties defaults = new Properties();
  static
  {
    defaults.setProperty(
      "model.factory", "be.ac.vub.cocompose.lang.impl.DefaultModelFactory");
    defaults.setProperty(
      "model.writer",  "be.ac.vub.cocompose.io.xmi.XMIModelWriter");
    defaults.setProperty(
      "model.reader",  "be.ac.vub.cocompose.io.xmi.XMIModelReader");

    defaults.setProperty(
      "debugging.enabled", "false");

    defaults.setProperty(
      "logging.log", "be.ac.vub.cocompose.log.DefaultLog");
  }
  private static CoCompose Singleton = new CoCompose();

  private Properties settings = null;
  private Log log = null;
  private ModelFactory factory = null;
  private Hashtable sharedModels = new Hashtable();
  private Hashtable sharedModelUsage = new Hashtable();
  private RemoveReferencedModels removeRef = new RemoveReferencedModels();

  /**
   * Runs a stand-alone CoCompose.
   * @param args command-line arguments
   */
  public static void main(String[] args)
  {
    // TODO stub
    System.out.println("Stand-alone CoCompose2 not yet implemented...");
  }

  /**
   * Gets the singleton CoCompose instance.
   * @return the singleton CoCompose instance.
   */
  public static CoCompose getDefault()
  {
    return Singleton;
  }
  
  /**
   * Private constructor for Singleton pattern.
   */
  private CoCompose()
  {
    defaults.setProperty(
      "model.factory", "be.ac.vub.cocompose.lang.impl.DefaultModelFactory");
    defaults.setProperty(
      "model.writer",  "be.ac.vub.cocompose.io.xmi.XMIModelWriter");
    defaults.setProperty(
      "model.reader",  "be.ac.vub.cocompose.io.xmi.XMIModelReader");

    defaults.setProperty(
      "debugging.enabled", "false");

    defaults.setProperty(
      "logging.log", "be.ac.vub.cocompose.log.DefaultLog");
    settings = new Properties(defaults);
    initSettings();
  }

  /**
   * Returns the standard logger.
   * @return the standard logger.
   */
  public Log getLog()
  {
    return log;
  }

  /**
   * Gets the model element factory
   * @return the factory
   */
  public ModelFactory getFactory()
  {
    return factory;
  }

  /**
   * Gets a global CoCompose setting
   * @param name the setting name
   * @return the setting value
   */
  public String getSetting(String name)
  {
    return settings.getProperty(name);
  }

  /**
   * Gets the global CoCompose settings.
   * @return the settings.
   */
  public Properties getSettings()
  {
    return settings;
  }

  /**
   * Sets the global CoCompose settings.
   * @param properties the settings.
   */
  public void setSettings(Properties properties)
  {
    settings = properties;
    initSettings();
  }

  /**
   * Adds a model to the set of shared models and increases the usage count. 
   * If the model is already shared, only the usage count is increased.
   * @param model the model to add.
   */
  public void addSharedModel(Model model)
  {
    Integer usage = (Integer) sharedModelUsage.get(model.getUri());
    if (usage == null)
    {
      sharedModels.put(model.getUri(), model);
      usage = new Integer(1);
    }
    else
    {
      usage = new Integer(usage.intValue() + 1);
    }
    sharedModelUsage.put(model.getUri(), usage);
  }

  /**
   * Removes a model from the set of shared models or decreases the usage count
   * if greater than 0.
   * @param model the model to remove.
   */
  public void removeSharedModel(Model model)
  {
    Integer usage = (Integer) sharedModelUsage.get(model.getUri());
    if (usage == null)
      return;
    if (usage.intValue() > 1)
    {
      usage = new Integer(usage.intValue() - 1);
      sharedModelUsage.put(model.getUri(), usage);
    }
    else
    {
      removeReferencedModels(model);
      sharedModels.remove(model.getUri());
      sharedModelUsage.remove(model.getUri());
    }
  }
  
  /**
   * Retrieves a shared model, given its URI.
   * @param uri the URI that indicates the model location.
   * @return the shared model or null if nothing found.
   */
  public Model getSharedModel(String uri)
  {
    return (Model) sharedModels.get(uri);
  }

  /**
   * Retrieves all shared models.
   * @return the shared models.
   */
  public Collection getSharedModels()
  {
    return sharedModels.values();
  }

  /**
   * (Re-)initialises this class after settings have been set.
   */
  private void initSettings()
  {
    try
    {
      log = (Log) Class.forName(
        getSetting("logging.log")).newInstance();
      log.setDebugging(
        settings.getProperty("debugging.enabled").equals("true"));
      factory = (ModelFactory) Class.forName(
        getSetting("model.factory")).newInstance();
    }
    catch (Exception e)
    {
      Exception we = new Exception(
        "Error: could not initialise CoCompose settings: " + e.getMessage(), 
        e);
      if (log == null)
        log = new DefaultLog();
      log.report(we);
    }
  }
  
  /**
   * Removes all shared models referenced from the given model.
   * @param model
   */
  private void removeReferencedModels(Model model)
  {
    try
    {
      removeRef.visit(model);
    }
    catch (ModelElementException e)
    {
      getLog().report(e);
    }
  }
}
