/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2003, 2004
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 * 
 * $Id: CoComposeImages.java,v 1.4 2004/05/10 15:13:53 dwagelaa Exp $
 * $Log: CoComposeImages.java,v $
 * Revision 1.4  2004/05/10 15:13:53  dwagelaa
 * + added the ModelTransformation language element
 * + added some more CVS Id tags
 * 0 refactored some classes to use Log
 *
 * Revision 1.3  2004/05/07 14:19:28  dwagelaa
 * + added model pool (shared model) support
 * + added inter-model reference support
 *
 * Revision 1.2  2004/04/30 16:24:55  dwagelaa
 * + added ElementConstraint support for the CoCompose2 editor
 *
 */

package be.ac.vub.cocompose.eclipse;

import java.net.MalformedURLException;
import java.net.URL;

import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.graphics.Image;

import be.ac.vub.cocompose.CoCompose;
import be.ac.vub.cocompose.lang.DefaultSuperModelVisitor;
import be.ac.vub.cocompose.lang.ModelElementException;
import be.ac.vub.cocompose.lang.constraints.Constraint;
import be.ac.vub.cocompose.lang.core.Concept;
import be.ac.vub.cocompose.lang.core.ImplementationGenerator;
import be.ac.vub.cocompose.lang.core.ImplementationPattern;
import be.ac.vub.cocompose.lang.core.ImplementationRolePart;
import be.ac.vub.cocompose.lang.core.ModelElement;
import be.ac.vub.cocompose.lang.core.ModelTransformation;
import be.ac.vub.cocompose.lang.core.Relationship;
import be.ac.vub.cocompose.lang.core.Role;
import be.ac.vub.cocompose.lang.core.SolutionPattern;
import be.ac.vub.cocompose.lang.properties.Property;

/**
 * Points to the images used for the CoCompose plugin.
 * @author Dennis Wagelaar
 * @version 0.2.10, 10/05/2004
 * @since 0.2.0
 */
public class CoComposeImages
{
  private final static URL BASE_URL = CoComposePlugin.getDefault().getDescriptor().getInstallURL();
  // paths
  private static final String PATH_ICONS = "icons/full/";
  private static final String PATH_OBJ    = PATH_ICONS + "ctool16/";
  private static final String PATH_WIZBAN = PATH_ICONS + "wizban/";
  // obj images
  public static final ImageDescriptor ELEMENT_OBJ      = createImgDesc(PATH_OBJ + "cocompose.gif");
  public static final ImageDescriptor CONCEPT_OBJ      = createImgDesc(PATH_OBJ + "concept.gif");
  public static final ImageDescriptor ROLE_OBJ         = createImgDesc(PATH_OBJ + "role.gif");
  public static final ImageDescriptor RELATIONSHIP_OBJ = createImgDesc(PATH_OBJ + "connection16.gif");
  public static final ImageDescriptor SOL_PATT_OBJ     = createImgDesc(PATH_OBJ + "sol_patt.gif");
  public static final ImageDescriptor IMPL_PATT_OBJ    = createImgDesc(PATH_OBJ + "impl_patt.gif");
  public static final ImageDescriptor IMPL_GEN_OBJ     = createImgDesc(PATH_OBJ + "impl_gen.gif");
  public static final ImageDescriptor IMPL_RP_OBJ      = createImgDesc(PATH_OBJ + "impl_role_part.gif");
  public static final ImageDescriptor MODEL_TRANS_OBJ  = createImgDesc(PATH_OBJ + "model_trans.gif");
  public static final ImageDescriptor PROPERTY_OBJ     = createImgDesc(PATH_OBJ + "property.gif");
  public static final ImageDescriptor CONSTRAINT_OBJ   = createImgDesc(PATH_OBJ + "constraint.gif");
  public static final ImageDescriptor OUTLINE_OBJ      = createImgDesc(PATH_OBJ + "outline.gif");
  public static final ImageDescriptor OVERVIEW_OBJ     = createImgDesc(PATH_OBJ + "overview.gif");
  // wizban image
  public static final ImageDescriptor NEWMODEL_WIZBAN = createImgDesc(PATH_WIZBAN + "newModel_wiz.gif");

  private static CoComposeImages Singleton = new CoComposeImages();

  /**
   * Determines the correct icon for each ModelElement.
   */
  class IconVisitor extends DefaultSuperModelVisitor
  {
    private ImageDescriptor icon = ELEMENT_OBJ;
    
    /**
     * Chooses CONCEPT_OBJ as icon.
     * @see be.ac.vub.cocompose.lang.ModelVisitor#visitConcept(be.ac.vub.cocompose.lang.core.Concept)
     */
    public void visitConcept(Concept concept) throws ModelElementException
    {
      icon = CONCEPT_OBJ;
    }

    /**
     * Chooses ROLE_OBJ as icon.
     * @see be.ac.vub.cocompose.lang.ModelVisitor#visitRole(be.ac.vub.cocompose.lang.core.Role)
     */    
    public void visitRole(Role role) throws ModelElementException
    {
      icon = ROLE_OBJ;
    }
    
    /**
     * Chooses RELATIONSHIP_OBJ as icon.
     * @see be.ac.vub.cocompose.lang.ModelVisitor#visitRelationship(be.ac.vub.cocompose.lang.core.Relationship)
     */
    public void visitRelationship(Relationship relationship) throws ModelElementException
    {
      icon = RELATIONSHIP_OBJ;
    }

    /**
     * Chooses SOL_PATT_OBJ as icon. 
     * @see be.ac.vub.cocompose.lang.ModelVisitor#visitSolutionPattern(be.ac.vub.cocompose.lang.core.SolutionPattern)
     */
    public void visitSolutionPattern(SolutionPattern solutionPattern)
    {
      icon = SOL_PATT_OBJ;
    }

    /**
     * Chooses IMPL_PATT_OBJ as icon. 
     * @see be.ac.vub.cocompose.lang.ModelVisitor#visitSolutionPattern(be.ac.vub.cocompose.lang.core.SolutionPattern)
     */    
    public void visitImplementationPattern(ImplementationPattern implementationPattern)
    {
      icon = IMPL_PATT_OBJ;
    }

    /**
     * Chooses IMPL_GEN_OBJ as icon. 
     * @see be.ac.vub.cocompose.lang.ModelVisitor#visitImplementationPattern(be.ac.vub.cocompose.lang.core.ImplementationPattern)
     */    
    public void visitImplementationGenerator(ImplementationGenerator implementationGenerator)
    {
      icon = IMPL_GEN_OBJ;
    }

    /**
     * Chooses IMPL_RP_OBJ as icon. 
     * @see be.ac.vub.cocompose.lang.ModelVisitor#visitImplementationRolePart(be.ac.vub.cocompose.lang.core.ImplementationRolePart)
     */
    public void visitImplementationRolePart(ImplementationRolePart implementationRolePart)
    {
      icon = IMPL_RP_OBJ;
    }

    /**
     * Chooses MODEL_TRANS_OBJ as icon. 
     * @see be.ac.vub.cocompose.lang.ModelVisitor#visitModelTransformation(be.ac.vub.cocompose.lang.core.ModelTransformation)
     */
    public void visitModelTransformation(ModelTransformation modelTransformation)
    {
      icon = MODEL_TRANS_OBJ;
    }

    /**
     * Chooses PROPERTY_OBJ as icon. 
     * @see be.ac.vub.cocompose.lang.ModelVisitor#visitImplementationRolePart(be.ac.vub.cocompose.lang.core.ImplementationRolePart)
     */
    public void visitProperty(Property property)
    {
      icon = PROPERTY_OBJ;
    }
    
    /**
     * Chooses a CONSTRAINT_OBJ as icon.
     * @see be.ac.vub.cocompose.lang.ModelVisitor#visitConstraint(be.ac.vub.cocompose.lang.constraints.Constraint)
     */
    public void visitConstraint(Constraint constraint)
    {
      icon = CONSTRAINT_OBJ;
    }

    /**
     * Gets the correct icon after visiting.
     * @return the icon.
     */
    public ImageDescriptor getIcon()
    {
      return icon;
    }
  }

  /**
   * Private constructor for CoComposeImages.
   */
  private CoComposeImages()
  {
  }

  /**
   * Returns the singleton.
   * @return the singleton CoComposeImages.
   */
  public static CoComposeImages getInstance()
  {
    return Singleton;
  }

  /**
   * Gets the correct icon for the given element.
   * @param element the element to get the icon for.
   * @return the icon.
   */
  public Image getElementIcon(ModelElement element)
  {
    IconVisitor icon = new IconVisitor();
    try
    {
      icon.visit(element);
    }
    catch (ModelElementException e)
    {
      CoCompose.getDefault().getLog().report(e);
    }
    return icon.getIcon().createImage();
  }

  /**
   * Creates an image descriptor.
   * @param path - the file path.
   * @return the image descriptor.
   */
  private static ImageDescriptor createImgDesc(String path) 
  {
    return ImageDescriptor.createFromURL(createURL(path));
  }

  /**
   * Creates a URL.
   * @param path - the file path.
   * @return the image descriptor.
   */
  private static URL createURL(String path) 
  {
    URL url = null;
    try 
    {
      url = new URL(BASE_URL, path);
    }
    catch (MalformedURLException mue) 
    {
      return null;
    }
    return url;
  }
}
