/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2003, 2004
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 * 
 * $Id: CoComposePlugin.java,v 1.5 2004/04/27 16:43:58 dwagelaa Exp $
 * $Log: CoComposePlugin.java,v $
 * Revision 1.5  2004/04/27 16:43:58  dwagelaa
 * + added drag-n-drop
 * + added copy/paste
 * + added general copying operation
 * - removed examples (in separate module now)
 *
 */

package be.ac.vub.cocompose.eclipse;

import java.io.File;
import java.io.FileInputStream;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.eclipse.core.runtime.IPluginDescriptor;
import org.eclipse.core.runtime.Platform;
import org.eclipse.gef.palette.CombinedTemplateCreationEntry;
import org.eclipse.gef.palette.ConnectionCreationToolEntry;
import org.eclipse.gef.palette.MarqueeToolEntry;
import org.eclipse.gef.palette.PaletteContainer;
import org.eclipse.gef.palette.PaletteDrawer;
import org.eclipse.gef.palette.PaletteGroup;
import org.eclipse.gef.palette.PaletteRoot;
import org.eclipse.gef.palette.PaletteSeparator;
import org.eclipse.gef.palette.SelectionToolEntry;
import org.eclipse.gef.palette.ToolEntry;
import org.eclipse.gef.requests.SimpleFactory;

import be.ac.vub.cocompose.CoCompose;
import be.ac.vub.cocompose.eclipse.model.core.EclipseConcept;
import be.ac.vub.cocompose.eclipse.model.core.EclipseRole;

/**
 * Eclipse plugin for CoCompose.
 * @author Dennis Wagelaar
 * @version 0.2.9, 27/04/2004
 * @since 0.2.0
 */
public class CoComposePlugin extends AbstractUIPlugin 
{
  private static final String SETTINGS_FILE = "cocompose.properties";

  private static CoComposePlugin Singleton;
  private static CoCompose Settings = CoCompose.getDefault();

  /**
   * Creates a CoComposePlugin and stores it as the singleton.
   * @param desc - Eclipse plugin descriptor.
   */
  public CoComposePlugin(IPluginDescriptor desc)
  {
    super(desc);
    if (Singleton == null)
    {
      Singleton = this;
    }
    try
    {
      URL settingsurl = new URL(getDescriptor().getInstallURL(), SETTINGS_FILE);
      settingsurl = Platform.asLocalURL(settingsurl);
      Properties settings = Settings.getSettings();
      settings.load(new FileInputStream(new File(settingsurl.getFile())));
      setDebugging(settings.getProperty("debugging.enabled").equals("true"));
      Settings.setSettings(settings);
    }
    catch (Exception e)
    { // may not happen
      System.err.println("Error: could not load CoCompose settings file " + SETTINGS_FILE);
      e.printStackTrace();
    }
  }

  /**
   * Returns the singleton.
   * @return the singleton CoComposePlugin.
   */
  public static CoComposePlugin getDefault()
  {
	  return Singleton;
  }
  
  /**
   * Creates the palette for the editor.
   * @return the palette root.
   */
  static PaletteRoot createPalette() 
  {
    PaletteRoot cocomposePalette = new PaletteRoot();
    cocomposePalette.addAll(createCategories(cocomposePalette));
    return cocomposePalette;
  }

  /**
   * Creates the editor palette categories with their elements.
   * @param root - the palette root.
   * @return the GUI elements.
   */
  static private List createCategories(PaletteRoot root)
  {
    List categories = new ArrayList();
	  categories.add(createControlGroup(root));
	  categories.add(createComponentsDrawer());
    return categories;
  }

  /**
   * Creates the control group for the palette.
   * @param root - the palette root.
   * @return the control group palette.
   */
  static private PaletteContainer createControlGroup(PaletteRoot root)
  {
    PaletteGroup controlGroup = new PaletteGroup("Control");

    List entries = new ArrayList();

    ToolEntry tool = new SelectionToolEntry();
    entries.add(tool);
    root.setDefaultEntry(tool);

    tool = new MarqueeToolEntry();
    entries.add(tool);
  
    PaletteSeparator sep = new PaletteSeparator("be.ac.vub.cocompose.sep2");
    sep.setUserModificationPermission(PaletteSeparator.PERMISSION_NO_MODIFICATION);
    entries.add(sep);

    tool = new ConnectionCreationToolEntry(
      "Relationship",
      "Relationship between CoCompose elements",
      null,
      CoComposeImages.RELATIONSHIP_OBJ,
      CoComposeImages.RELATIONSHIP_OBJ
    );
    entries.add(tool);

    controlGroup.addAll(entries);
    return controlGroup;
  }

  /**
   * Creates the component group for the palette.
   * @return the component group palette.
   */
  static private PaletteContainer createComponentsDrawer()
  {
    PaletteDrawer drawer = new PaletteDrawer("Elements", CoComposeImages.ELEMENT_OBJ);

    List entries = new ArrayList();
  
    CombinedTemplateCreationEntry combined = new CombinedTemplateCreationEntry(
      "Concept",
      "CoCompose concept language element",
      TemplateConstants.TEMPLATE_CONCEPT,
      new SimpleFactory(EclipseConcept.class),
      CoComposeImages.CONCEPT_OBJ,
      CoComposeImages.CONCEPT_OBJ
    );
    entries.add(combined);

    combined = new CombinedTemplateCreationEntry(
      "Role",
      "CoCompose role language element",
      TemplateConstants.TEMPLATE_ROLE,
      new SimpleFactory(EclipseRole.class),
      CoComposeImages.ROLE_OBJ,
      CoComposeImages.ROLE_OBJ
    );
    entries.add(combined);

    drawer.addAll(entries);
    return drawer;
  }
}
