/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2003, 2004
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 * 
 * $Id: CreateCommand.java,v 1.5 2004/05/18 11:40:13 dwagelaa Exp $
 * $Log: CreateCommand.java,v $
 * Revision 1.5  2004/05/18 11:40:13  dwagelaa
 * 0 worked on EPS and SVG export: both work now, except that images aren't supported in SVG
 * 0 worked on ApplySolutionPattern: the result is now properly displayed in the editor, and undoable ;-)
 *
 * Revision 1.4  2004/05/17 14:02:54  dwagelaa
 * + added initial ApplySolutionPattern functionality
 * 0 changed EPSGraphics to AWTGraphics
 * + added initial SVG export functionality (EPS works by now)
 * 0 updated version numbers to 0.2.11
 *
 * Revision 1.3  2004/04/28 12:24:36  dwagelaa
 * worked on drag-n-drop
 * updated copy/paste
 * updated deep copy operation
 *
 */

package be.ac.vub.cocompose.eclipse.commands;

import java.awt.Point;
import java.util.List;

import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.commands.Command;

import be.ac.vub.cocompose.CoCompose;
import be.ac.vub.cocompose.lang.ModelElementException;
import be.ac.vub.cocompose.lang.core.ModelElement;
import be.ac.vub.cocompose.lang.core.Namespace;
import be.ac.vub.cocompose.lang.core.RelationElement;
import be.ac.vub.cocompose.log.Log;

/**
 * Create command for CoCompose elements.
 * @author Dennis Wagelaar
 * @version 0.2.11, 17/05/2004
 * @since 0.2.0
 */
public class CreateCommand extends Command
{
  private Log log = CoCompose.getDefault().getLog();
  private ModelElement element = null;
  private Namespace owner = null;
  private int index = -1;
  private Namespace oldOwner = null;
  private String oldName = null;
  private String oldId = null;
  private int oldIndex = -1;
  private Rectangle position = null;

  /**
   * Creates a CreateCommand.
   */
  public CreateCommand() 
  {
    super("Create");
  }

  /**
   * @see org.eclipse.gef.commands.Command#execute()
   */
  public void execute() 
  {
    primExecute();
  }

  /**
   * Gets the owner element.
   * @return the owner element.
   */
  public Namespace getOwner()
  {
    return owner;
  }

  /**
   * Sets the owner element.
   * @param owner - the owner element.
   */
  public void setOwner(Namespace owner)
  {
    this.owner = owner;
  }

  /**
   * Gets the created element.
   * @return the created element.
   */
  public ModelElement getElement()
  {
    return element;
  }

  /**
   * Sets the created element.
   * @param element - the created element.
   */
  public void setElement(ModelElement element)
  {
    this.element = element;
  }

  /**
   * Gets the position for insertion.
   * @return the position data.
   */
  public Rectangle getPosition()
  {
    return position;
  }

  /**
   * Sets the position for insertion.
   * @param position - the position data.
   */
  public void setPosition(Rectangle position)
  {
    this.position = position;
  }

  /**
   * @see org.eclipse.gef.commands.Command#redo()
   */
  public void redo() 
  {
    primExecute();
  }

  /**
   * @see org.eclipse.gef.commands.Command#undo()
   */
  public void undo() 
  {
    owner.removeOwnedElement(element);
    if (oldOwner != null)
    {
      try
      {
        element.setNamespace(null);
        element.setId(oldId);
        element.setName(oldName);
        oldOwner.addOwnedElement(oldIndex, element);
      }
      catch (ModelElementException e)
      {
        log.report(e);
      }
    }
  }

  /**
   * Execute implementation.
   * @see org.eclipse.gef.commands.Command#execute()
   */
  protected void primExecute()
  {
    oldId = element.getId();
    oldName = element.getName();
    oldOwner = element.getNamespace();
    if (oldOwner != null)
    {
      oldIndex = oldOwner.getOwnedElements().indexOf(element);
      oldOwner.removeOwnedElement(element);
    }
    if ((position != null) && (element instanceof RelationElement)) 
    {
      RelationElement relel = (RelationElement) element;
      relel.setPosition(new Point(position.x, position.y));
    }
    try
    {
      element.setNamespace(null);
      element.setId(adaptId(oldId));
      if (element instanceof Namespace)
      {
        Namespace ns = (Namespace) element;
        List ownedElts = ns.getDeepOwnedElements(ModelElement.class);
        for (int index = 0; index < ownedElts.size(); index++)
        {
          ModelElement owned = (ModelElement) ownedElts.get(index);
          owned.setId(adaptId(owned.getId()));
        }
      }
      element.setName(adaptName(oldName));
      owner.addOwnedElement(index, element);
    }
    catch (ModelElementException e)
    {
      log.report(e);
    }
  }

  /**
   * Adapts the given id to be unique within the new namespace. If the given
   * id is not unique, null will be returned so that the owner can generate
   * an id later.
   * @param id the id to adapt
   * @return the adapted id
   */  
  protected String adaptId(String id)
  {
    if (id != null)
    {
      if (!owner.getOwnedIds().containsKey(id))
        return id;
    }
    return null;
  }
  
  /**
   * Adapts the given name to be unique within the new namespace.
   * @param name the name to adapt
   * @return the adapted name
   */  
  protected String adaptName(String name)
  {
    if (name != null)
    {
      if (!owner.isRegistered(name))
        return name;
      int suffix = 1;
      while (owner.isRegistered(name + suffix))
        suffix++;
      return name + suffix;
    }
    return null;
  }
  
  /**
   * Gets the insertion index.
   * @return the insertion index.
   */
  public int getIndex()
  {
    return index;
  }

  /**
   * Sets the insertion index.
   * @param i the insertion index.
   */
  public void setIndex(int i)
  {
    index = i;
  }
}
