/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2004
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 * Copyright portions (c) 2000, 2003 IBM Corporation and others.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 * 
 * $Id: AWTGraphics.java,v 1.2 2004/05/18 11:40:13 dwagelaa Exp $
 * $Log: AWTGraphics.java,v $
 * Revision 1.2  2004/05/18 11:40:13  dwagelaa
 * 0 worked on EPS and SVG export: both work now, except that images aren't supported in SVG
 * 0 worked on ApplySolutionPattern: the result is now properly displayed in the editor, and undoable ;-)
 *
 * Revision 1.1  2004/05/17 14:02:54  dwagelaa
 * + added initial ApplySolutionPattern functionality
 * 0 changed EPSGraphics to AWTGraphics
 * + added initial SVG export functionality (EPS works by now)
 * 0 updated version numbers to 0.2.11
 *
 * Revision 1.3  2004/04/28 16:30:08  dwagelaa
 * rearranged EPS export under ExportAction
 *
 * Revision 1.2  2004/04/26 16:30:43  dwagelaa
 * Added CVS tags
 *
 */

package be.ac.vub.cocompose.eclipse.draw2d;

import java.awt.BasicStroke;
import java.awt.Graphics2D;
import java.awt.Polygon;
import java.awt.Stroke;
import java.awt.image.BufferedImage;
import java.util.Stack;

import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.PointList;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.FontMetrics;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.widgets.Display;

/**
 * Glue Graphics class for AWT. Code portions taken from
 * {@link org.eclipse.draw2d.SWTGraphics}.
 * @author Dennis Wagelaar
 * @author IBM Corporation and others
 * @version 0.2.11, 18/05/2004
 * @since 0.2.9
 * @see org.eclipse.draw2d.SWTGraphics
 */
public class AWTGraphics extends Graphics
{
  private static final float[] DASH        = {5};
  private static final float[] DOT         = {1};
  private static final float[] DASHDOT     = {5, 1};
  private static final float[] DASHDOTDOT  = {5, 1, 1};
  
  private static final boolean FILL_MODE = true;
  private static final boolean DRAW_MODE = true;
  
  protected class State implements Cloneable
  {
    public Color bgColor, fgColor;
    public int clipH, clipW, clipX, clipY, dx, dy, lineStyle, lineWidth;
    public Font font;
    public boolean xor;
    
    /**
     * Creates a new State.
     */
    protected State()
    {
    }
    
    /**
     * @see java.lang.Object#clone()
     */
    public Object clone() throws CloneNotSupportedException
    {
      return super.clone();
    }

    /**
     * Copies all state information from the given State to this State.
     * @param state The State to copy from.
     */
    public void copyFrom(State state) 
    {
      bgColor = state.bgColor;
      fgColor = state.fgColor;
      clipH = state.clipH;
      clipW = state.clipW;
      clipX = state.clipX;
      clipY = state.clipY;
      dx = state.dx;
      dy = state.dy;
      lineStyle = state.lineStyle;
      lineWidth = state.lineWidth;
      font = state.font;
      xor = state.xor;
    }
  }
  
  private Graphics2D inner;
  private GC gc = new GC(Display.getDefault());
  private Stack stateStack = new Stack();
  private State currentState = new State();
  private State appliedState = new State();
  private Rectangle relativeClip;
  private int translateX = 0, translateY = 0;
  private float fontSizeDelta = 1;

  /**
   * Creates an AWTGraphics.
   * @param inner the inner AWT graphics object.
   */
  public AWTGraphics(Graphics2D inner)
  {
    super();
    if (inner == null)
      throw new Error("Inner Graphics2D object may not be null");
    this.inner = inner;
    relativeClip = new Rectangle(gc.getClipping());
    currentState.bgColor = appliedState.bgColor = gc.getBackground();
    inner.setBackground(toAWTColor(currentState.bgColor));
    currentState.fgColor = appliedState.fgColor = gc.getForeground();
    inner.setColor(toAWTColor(currentState.fgColor));
    setInnerFont(currentState.font = appliedState.font = gc.getFont());
    setInnerLineWidth(currentState.lineWidth = appliedState.lineWidth = gc.getLineWidth());
    setInnerLineStyle(currentState.lineStyle = appliedState.lineStyle = gc.getLineStyle());
    currentState.clipX = appliedState.clipX = relativeClip.x;
    currentState.clipY = appliedState.clipY = relativeClip.y;
    currentState.clipW = appliedState.clipW = relativeClip.width;
    currentState.clipH = appliedState.clipH = relativeClip.height;
    currentState.xor = appliedState.xor = gc.getXORMode();
  }

  /**
   * @see org.eclipse.draw2d.Graphics#clipRect(org.eclipse.draw2d.geometry.Rectangle)
   */
  public void clipRect(Rectangle r)
  {
    relativeClip.intersect(r);
    setClipAbsolute(
      relativeClip.x + translateX,
      relativeClip.y + translateY,
      relativeClip.width,
      relativeClip.height);
  }

  /**
   * @see org.eclipse.draw2d.Graphics#dispose()
   */
  public void dispose()
  {
    stateStack.clear();
    inner.dispose();
    gc.dispose();
  }

  /**
   * @see org.eclipse.draw2d.Graphics#drawArc(int, int, int, int, int, int)
   */
  public void drawArc(int x, int y, int w, int h, int offset, int length)
  {
    checkPaint();
    inner.drawArc(x + translateX, y + translateY, w + 1, h + 1, offset, length);
  }

  /**
   * @see org.eclipse.draw2d.Graphics#fillArc(int, int, int, int, int, int)
   */
  public void fillArc(int x, int y, int w, int h, int offset, int length)
  {
    checkFill();
    inner.fillArc(x + translateX, y + translateY, w + 1, h + 1, offset, length);
  }

  /**
   * @see org.eclipse.draw2d.Graphics#fillGradient(int, int, int, int, boolean)
   */
  public void fillGradient(int x, int y, int w, int h, boolean vertical)
  { // closest match
    checkFill();
    inner.fill3DRect(x + translateX, y + translateY, w + 1, h + 1, vertical);
  }

  /**
   * @see org.eclipse.draw2d.Graphics#drawFocus(int, int, int, int)
   */
  public void drawFocus(int x, int y, int w, int h)
  { // closest match
    checkPaint();
    inner.drawRect(x + translateX, y + translateY, w + 1, h + 1);
  }

  /**
   * Not yet implemented.
   * @see org.eclipse.draw2d.Graphics#drawImage(org.eclipse.swt.graphics.Image, int, int)
   */
  public void drawImage(Image srcImage, int x, int y)
  {
    // inner will first fill a rectangle before drawing the image, so make sure
    // that box will have the right color.
    checkFill();
    inner.setBackground(inner.getColor());
    inner.drawImage(toAWTImage(srcImage), x + translateX, y + translateY, null);
  }

  /**
   * Not yet implemented.
   * @see org.eclipse.draw2d.Graphics#drawImage(org.eclipse.swt.graphics.Image, int, int, int, int, int, int, int, int)
   */
  public void drawImage(
    Image srcImage,
    int x1,
    int y1,
    int w1,
    int h1,
    int x2,
    int y2,
    int w2,
    int h2)
  {
    // inner will first fill a rectangle before drawing the image, so make sure
    // that box will have the right color.
    checkFill();
    inner.setBackground(inner.getColor());
    inner.drawImage(
      toAWTImage(srcImage),
      x2 + translateX,
      y2 + translateY,
      x2 + w2 + translateX,
      y2 + h2 + translateY,
      x1 + translateX,
      y1 + translateY,
      x1 + w1 + translateX,
      y1 + h1 + translateX,
      null);
  }

  /**
   * @see org.eclipse.draw2d.Graphics#drawLine(int, int, int, int)
   */
  public void drawLine(int x1, int y1, int x2, int y2)
  {
    checkPaint();
    inner.drawLine(x1 + translateX, y1 + translateY, x2 + translateX, y2 + translateY);
  }

  /**
   * @see org.eclipse.draw2d.Graphics#drawOval(int, int, int, int)
   */
  public void drawOval(int x, int y, int w, int h)
  {
    checkPaint();
    inner.drawOval(x + translateX, y + translateY, w + 1, h + 1);
  }

  /**
   * @see org.eclipse.draw2d.Graphics#fillOval(int, int, int, int)
   */
  public void fillOval(int x, int y, int w, int h)
  {
    checkFill();
    inner.fillOval(x + translateX, y + translateY, w + 1, h + 1);
  }

  /**
   * @see org.eclipse.draw2d.Graphics#drawPolygon(org.eclipse.draw2d.geometry.PointList)
   */
  public void drawPolygon(PointList points)
  {
    checkPaint();
    try
    {
      points.translate(translateX, translateY);
      inner.drawPolygon(toPolygon(points));
    }
    finally
    {
      points.translate(-translateX, -translateY);
    }
  }

  /**
   * @see org.eclipse.draw2d.Graphics#fillPolygon(org.eclipse.draw2d.geometry.PointList)
   */
  public void fillPolygon(PointList points)
  {
    checkFill();
    try
    {
      points.translate(translateX, translateY);
      inner.fillPolygon(toPolygon(points));
    }
    finally
    {
      points.translate(-translateX, -translateY);
    }
  }

  /**
   * @see org.eclipse.draw2d.Graphics#drawPolyline(org.eclipse.draw2d.geometry.PointList)
   */
  public void drawPolyline(PointList points)
  {
    checkPaint();
    try 
    {
      points.translate(translateX, translateY);
      int nPoints = points.size();
      int[] pts = points.toIntArray();
      int[] xPoints = new int[nPoints];
      int[] yPoints = new int[nPoints];
      for (int i = 0; i < nPoints; i++)
      {
        xPoints[i] = pts[i*2];
        yPoints[i] = pts[i*2+1];
      }
      inner.drawPolyline(xPoints, yPoints, nPoints);
    } 
    finally
    {
      points.translate(-translateX, -translateY);
    }
  }

  /**
   * @see org.eclipse.draw2d.Graphics#drawRectangle(int, int, int, int)
   */
  public void drawRectangle(int x, int y, int width, int height)
  {
    checkPaint();
    inner.drawRect(x + translateX, y + translateY, width + 1, height + 1);
  }

  /**
   * @see org.eclipse.draw2d.Graphics#fillRectangle(int, int, int, int)
   */
  public void fillRectangle(int x, int y, int width, int height)
  {
    checkFill();
    inner.fillRect(x + translateX, y + translateY, width + 1, height + 1);
  }

  /**
   * @see org.eclipse.draw2d.Graphics#drawRoundRectangle(org.eclipse.draw2d.geometry.Rectangle, int, int)
   */
  public void drawRoundRectangle(Rectangle r, int arcWidth, int arcHeight)
  {
    checkPaint();
    inner.drawRoundRect(
      r.x + translateX,
      r.y + translateY,
      r.width + 1,
      r.height + 1,
      arcWidth,
      arcHeight);
  }

  /**
   * @see org.eclipse.draw2d.Graphics#fillRoundRectangle(org.eclipse.draw2d.geometry.Rectangle, int, int)
   */
  public void fillRoundRectangle(Rectangle r, int arcWidth, int arcHeight)
  {
    checkFill();
    inner.fillRoundRect(
      r.x + translateX,
      r.y + translateY,
      r.width + 1,
      r.height + 1,
      arcWidth,
      arcHeight);
  }

  /**
   * @see org.eclipse.draw2d.Graphics#drawText(java.lang.String, int, int)
   */
  public void drawText(String s, int x, int y)
  {
    checkText();
    // always expands special characters
    inner.drawString(s, x + translateX, transposeTextY(y + translateY));
  }

  /**
   * @see org.eclipse.draw2d.Graphics#drawString(java.lang.String, int, int)
   */
  public void drawString(String s, int x, int y)
  {
    checkText();
    // always expands special characters
    inner.drawString(s, x + translateX, transposeTextY(y + translateY));
  }

  /**
   * @see org.eclipse.draw2d.Graphics#fillString(java.lang.String, int, int)
   */
  public void fillString(String s, int x, int y)
  {
    checkText();
    // closest match
    // always expands special characters
    inner.drawString(s, x + translateX, transposeTextY(y + translateY));
  }

  /**
   * @see org.eclipse.draw2d.Graphics#fillText(java.lang.String, int, int)
   */
  public void fillText(String s, int x, int y)
  {
    checkText();
    // closest match
    // always expands special characters
    inner.drawString(s, x + translateX, transposeTextY(y + translateY));
  }

  /**
   * @see org.eclipse.draw2d.Graphics#getBackgroundColor()
   */
  public Color getBackgroundColor()
  {
    return currentState.bgColor;
  }

  /**
   * @see org.eclipse.draw2d.Graphics#getClip(org.eclipse.draw2d.geometry.Rectangle)
   */
  public Rectangle getClip(Rectangle rect)
  {
    rect.setBounds(relativeClip);
    return rect;
  }

  /**
   * @see org.eclipse.draw2d.Graphics#getFont()
   */
  public Font getFont()
  {
    return currentState.font;
  }

  /**
   * @see org.eclipse.draw2d.Graphics#getFontMetrics()
   */
  public FontMetrics getFontMetrics()
  {
    checkText();
    return gc.getFontMetrics();
  }

  /**
   * @see org.eclipse.draw2d.Graphics#getForegroundColor()
   */
  public Color getForegroundColor()
  {
    return currentState.fgColor;
  }

  /**
   * @see org.eclipse.draw2d.Graphics#getLineStyle()
   */
  public int getLineStyle()
  {
    return currentState.lineStyle;
  }

  /**
   * @see org.eclipse.draw2d.Graphics#getLineWidth()
   */
  public int getLineWidth()
  {
    return currentState.lineWidth;
  }

  /**
   * @see org.eclipse.draw2d.Graphics#getXORMode()
   */
  public boolean getXORMode()
  {
    return currentState.xor;
  }

  /**
   * @see org.eclipse.draw2d.Graphics#popState()
   */
  public void popState()
  {
    restoreState((State) stateStack.pop());
  }

  /**
   * @see org.eclipse.draw2d.Graphics#pushState()
   */
  public void pushState()
  {
    try
    {
      stateStack.push(currentState.clone());
    }
    catch (CloneNotSupportedException e)
    {
      throw new RuntimeException(e.getMessage());
    }
  }

  /**
   * @see org.eclipse.draw2d.Graphics#restoreState()
   */
  public void restoreState()
  {
    restoreState((State) stateStack.peek());
  }

  /**
   * Ignored.
   * @see org.eclipse.draw2d.Graphics#scale(double)
   */
  public void scale(double amount)
  {
  }

  /**
   * @see org.eclipse.draw2d.Graphics#setBackgroundColor(org.eclipse.swt.graphics.Color)
   */
  public void setBackgroundColor(Color rgb)
  {
    if (currentState.bgColor.equals(rgb))
      return;
    currentState.bgColor = rgb;
  }

  /**
   * @see org.eclipse.draw2d.Graphics#setClip(org.eclipse.draw2d.geometry.Rectangle)
   */
  public void setClip(Rectangle r)
  {
    relativeClip.x = r.x;
    relativeClip.y = r.y;
    relativeClip.width = r.width;
    relativeClip.height = r.height;

    setClipAbsolute(
      r.x + translateX,
      r.y + translateY,
      r.width,
      r.height);
  }

  /**
   * @see org.eclipse.draw2d.Graphics#setFont(org.eclipse.swt.graphics.Font)
   */
  public void setFont(Font f)
  {
    if (currentState.font == f) 
      return;
    currentState.font = f;
  }

  /**
   * @see org.eclipse.draw2d.Graphics#setForegroundColor(org.eclipse.swt.graphics.Color)
   */
  public void setForegroundColor(Color rgb)
  {
    if (currentState.fgColor.equals(rgb))
      return;
    currentState.fgColor = rgb;
  }

  /**
   * @see org.eclipse.draw2d.Graphics#setLineStyle(int)
   */
  public void setLineStyle(int style)
  {
    if (currentState.lineStyle == style)
      return;
    currentState.lineStyle = style;
  }

  /**
   * @see org.eclipse.draw2d.Graphics#setLineWidth(int)
   */
  public void setLineWidth(int width)
  {
    if (currentState.lineWidth == width)
      return;
    currentState.lineWidth = width;
  }

  /**
   * Currently ignored.
   * @see org.eclipse.draw2d.Graphics#setXORMode(boolean)
   */
  public void setXORMode(boolean b)
  {
    // ignored
  }

  /**
   * @see org.eclipse.draw2d.Graphics#translate(int, int)
   */
  public void translate(int dx, int dy)
  {
    setTranslation(translateX + dx, translateY + dy);
    relativeClip.x -= dx;
    relativeClip.y -= dy;
  }
  
  /**
   * Gets the font size delta that should be propagated to the inner Graphics2D.
   * Default is 1.0
   * @return the font size delta.
   */
  public float getFontSizeDelta()
  {
    return fontSizeDelta;
  }

  /**
   * Sets the font size delta that should be propagated to the inner Graphics2D.
   * Default is 1.0
   * @param f the font size delta.
   */
  public void setFontSizeDelta(float f)
  {
    fontSizeDelta = f;
  }
  
  /**
   * Returns the EPS content of this Graphics object.
   * @see java.lang.Object#toString()
   */
  public String toString()
  {
    return inner.toString();
  }

  /**
   * Converts a PointList to a Polygon.
   * @param points the point list to convert.
   * @return the polygon representation of the point list.
   */
  protected Polygon toPolygon(PointList points)
  {
    Polygon pol = new Polygon();
    for (int i = 0; i < points.size(); i++)
    {
      Point pt = points.getPoint(i);
      pol.addPoint(pt.x, pt.y);
    }
    return pol;
  }
  
  /**
   * Converts an Eclipse/SWT Image to an AWT Image.
   * @param srcImage the source SWT image.
   * @return the converted AWT image.
   */
  protected BufferedImage toAWTImage(Image srcImage)
  {
    srcImage.setBackground(getBackgroundColor());
    ImageData src = srcImage.getImageData();
    BufferedImage awtImage =
      new BufferedImage(src.width, src.height, BufferedImage.TYPE_INT_RGB);

    int[] pixels = new int[src.width * src.height];
    for (int y = src.y; y < src.height; y++)
    {
      src.getPixels(src.x, y, src.width, pixels, y * src.width);
    }

    RGB rgb = getBackgroundColor().getRGB();
    // check transparency through alpha channel
    for (int y = src.y; y < src.height; y++)
    {
      for (int x = src.x; x < src.width; x++)
      {
        if (src.getAlpha(x, y) == 0)
        { // transparent pixel
          pixels[y * src.width + x] =
            (255 << 24) | (rgb.red << 16) | (rgb.green << 8) | rgb.blue;
        }
      }
    }
    // check transparency through transparentPixel
    if (src.transparentPixel != -1)
    {
      for (int i = 0; i < pixels.length; i++)
      {
        if (pixels[i] == src.transparentPixel)
        { // transparent pixel
          pixels[i] =
            (255 << 24) | (rgb.red << 16) | (rgb.green << 8) | rgb.blue;
        }
      }
    }
    // check transparency through mask
    ImageData mask = src.getTransparencyMask();
    if (mask != null)
    {
      for (int y = mask.y; y < mask.height; y++)
      {
        for (int x = mask.x; x < mask.width; x++)
        {
          if (mask.getPixel(x, y) == 0)
          { // transparent pixel
            pixels[y * mask.width + x] =
              (255 << 24) | (rgb.red << 16) | (rgb.green << 8) | rgb.blue;
          }
        }
      }
    }

    awtImage.setRGB(0, 0, src.width, src.height, pixels, 0, src.width);
    return awtImage;
  }
  
  /**
   * Converts an SWT Color to an AWT Color.
   * @param rgb the SWT Color.
   * @return the AWT Color.
   */
  protected java.awt.Color toAWTColor(Color rgb)
  {
    return new java.awt.Color(rgb.getRed(), rgb.getGreen(), rgb.getBlue());
  }
  
  /**
   * Converts an AWT Color to an SWT Color.
   * @param rgb the AWT Color.
   * @return the SWT Color.
   */
  protected Color toSWTColor(java.awt.Color rgb)
  {
    return new Color(Display.getDefault(), rgb.getRed(), rgb.getGreen(), rgb.getBlue());
  }
  
  /**
   * Transpose y coordinate by font height.
   * @param y the y coordinate to transpose.
   * @return the transposed y coordinate.
   */
  protected int transposeTextY(int y)
  {
    return y + getFontMetrics().getAscent();
  }

  /**
   * Sets the linestyle of the inner EpsGraphics.
   * @param style the line style.
   */
  protected void setInnerLineStyle(int style)
  {
    Stroke stroke = inner.getStroke();
    if (stroke instanceof BasicStroke)
    {
      BasicStroke bs = (BasicStroke) stroke;
      BasicStroke nbs;
      float[] dash = null; // SOLID
      if (SWT.LINE_DASH == style)
        dash = DASH;
      else if (SWT.LINE_DOT == style)
        dash = DOT;
      else if (SWT.LINE_DASHDOT == style)
        dash = DASHDOT;
      else if (SWT.LINE_DASHDOTDOT == style)
        dash = DASHDOTDOT;
      nbs =
        new BasicStroke(
          bs.getLineWidth(),
          bs.getEndCap(),
          bs.getLineJoin(),
          bs.getMiterLimit(),
          dash,
          bs.getDashPhase());
      inner.setStroke(nbs);
    }
  }

  /**
   * Sets the line width of the inner EpsGraphics.
   * @param width the line width.
   */
  protected void setInnerLineWidth(int width)
  {
    Stroke stroke = inner.getStroke();
    if (stroke instanceof BasicStroke)
    {
      BasicStroke bs = (BasicStroke) stroke;
      BasicStroke nbs =
        new BasicStroke(
          width,
          bs.getEndCap(),
          bs.getLineJoin(),
          bs.getMiterLimit(),
          bs.getDashArray(),
          bs.getDashPhase());
      inner.setStroke(nbs);
    }
  }

  /**
   * Sets the font of the inner EpsGraphics.
   * @param f the font to set.
   */
  protected void setInnerFont(Font f)
  {
    FontData[] fd = f.getFontData();
    inner.setFont(
      new java.awt.Font(
        fd[0].getName(),
        fd[0].getStyle(),
        Math.round(fd[0].getHeight() * getFontSizeDelta())));
  }

  /**
   * Sets clip values to the given values.
   * @param x the X value
   * @param y the Y value
   * @param w the width value
   * @param h the height value
   */
  protected void setClipAbsolute(int x, int y, int w, int h)
  {
    if (currentState.clipW == w
     && currentState.clipH == h
     && currentState.clipX == x
     && currentState.clipY == y)
      return;

    currentState.clipX = x;
    currentState.clipY = y;
    currentState.clipW = w;
    currentState.clipH = h;
  }
  
  /**
   * If the XOR or the clip region has change, these changes will be pushed to the GC.
   */
  protected void checkGC()
  {
    if (appliedState.xor != currentState.xor)
      gc.setXORMode(appliedState.xor = currentState.xor);
    if (appliedState.clipX != currentState.clipX
      || appliedState.clipY != currentState.clipY
      || appliedState.clipW != currentState.clipW
      || appliedState.clipH != currentState.clipH)
    {
      gc.setClipping(
        appliedState.clipX = currentState.clipX,
        appliedState.clipY = currentState.clipY,
        appliedState.clipW = currentState.clipW,
        appliedState.clipH = currentState.clipH);
        // don't clip, it causes mayhem for SVG output and degenerates EPS
        // output (i.e. in my testcases).
//      inner.setClip(
//        currentState.clipX,
//        currentState.clipY,
//        currentState.clipW,
//        currentState.clipH);
    }
  }
  
  /**
   * If the line width, line style, foreground or background colors have changed, these
   * changes will be pushed to the GC.  Also calls {@link #checkGC()}.
   */
  protected void checkPaint()
  {
    checkGC();
    if (!appliedState.fgColor.equals(currentState.fgColor))
    {
      gc.setForeground(appliedState.fgColor = currentState.fgColor);
    }
    inner.setColor(toAWTColor(currentState.fgColor));
    if (appliedState.lineStyle != currentState.lineStyle)
    {
      gc.setLineStyle(appliedState.lineStyle = currentState.lineStyle);
    }
    setInnerLineStyle(currentState.lineStyle);
    if (appliedState.lineWidth != currentState.lineWidth)
    {
      gc.setLineWidth(appliedState.lineWidth = currentState.lineWidth);
    }
    setInnerLineWidth(currentState.lineWidth);
  }

  /**
   * If the background color has changed, this change will be pushed to the GC.  Also calls
   * {@link #checkGC()}.
   */
  protected void checkFill()
  {
    checkGC();
    if (!appliedState.fgColor.equals(currentState.bgColor))
    {
      gc.setBackground(appliedState.fgColor = currentState.bgColor);
    }
    inner.setBackground(toAWTColor(currentState.bgColor));
    inner.setColor(toAWTColor(currentState.bgColor));
  }
  
  /**
   * If the font has changed, this change will be pushed to the GC.  Also calls
   * {@link #checkPaint()}.
   */
  protected final void checkText()
  {
    checkPaint();
    if (!appliedState.font.equals(currentState.font))
    {
      gc.setFont(appliedState.font = currentState.font);
    }
    setInnerFont(currentState.font);
  }

  /**
   * Sets all State information to that of the given State, called by restoreState()
   * @param s the State
   */
  protected void restoreState(State s)
  {
    setBackgroundColor(s.bgColor);
    setForegroundColor(s.fgColor);
    setLineStyle(s.lineStyle);
    setLineWidth(s.lineWidth);
    setFont(s.font);
    setXORMode(s.xor);
    setClipAbsolute(s.clipX, s.clipY, s.clipW, s.clipH);

    translateX = currentState.dx = s.dx;
    translateY = currentState.dy = s.dy;

    relativeClip.x = s.clipX - translateX;
    relativeClip.y = s.clipY - translateY;
    relativeClip.width = s.clipW;
    relativeClip.height = s.clipH;
  }

  /**
   * Sets the translation values of this to the given values
   * @param x The x value
   * @param y The y value
   */
  protected void setTranslation(int x, int y)
  {
    translateX = currentState.dx = x;
    translateY = currentState.dy = y;
  }
}
