/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2003, 2004
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 * 
 * $Id: ModelElementsPropertySource.java,v 1.3 2004/05/10 14:26:23 dwagelaa Exp $
 * $Log: ModelElementsPropertySource.java,v $
 * Revision 1.3  2004/05/10 14:26:23  dwagelaa
 * worked on shared models and eclipse property sources
 *
 * Revision 1.2  2004/05/07 14:19:28  dwagelaa
 * + added model pool (shared model) support
 * + added inter-model reference support
 *
 * Revision 1.1  2004/04/30 16:24:55  dwagelaa
 * + added ElementConstraint support for the CoCompose2 editor
 *
 */

package be.ac.vub.cocompose.eclipse.model.datatypes;

import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import org.eclipse.core.internal.runtime.Assert;
import org.eclipse.ui.views.properties.ComboBoxPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertySource;

import be.ac.vub.cocompose.CoCompose;
import be.ac.vub.cocompose.lang.core.Model;
import be.ac.vub.cocompose.lang.core.ModelElement;
import be.ac.vub.cocompose.lang.core.Namespace;
import be.ac.vub.cocompose.log.Log;

/**
 * Eclipse property source for multiple ModelElements.
 * @author Dennis Wagelaar
 * @version 0.2.10, 10/05/2004
 * @since 0.2.10
 */
public abstract class ModelElementsPropertySource implements IPropertySource
{
  private CoCompose settings = CoCompose.getDefault();
  private ModelElement subject = null; 
  private List values = null;
  private IPropertyDescriptor[] desc = null;
  private List possValues = null;
  private List valueStrings = null;
  
  /**
   * Creates a PartOfPropertySource.
   * @param subject the subject for which to set the property.
   * @param values the current values of the property.
   * @param namespace the Namespace in which to look for the property.
   * @param elementClass the class of the possible value elements.
   */
  public ModelElementsPropertySource(
    ModelElement subject,
    List values,
    Namespace namespace,
    Class elementClass)
  {
    Assert.isNotNull(subject);
    Assert.isNotNull(values);
    this.subject = subject;
    this.values = values;
    createPossibleValues(elementClass, namespace);
    createValueStrings();
  }

  /**
   * Creates a PartOfPropertySource. Looks in all models that are currently
   * loaded in {@link be.ac.vub.cocompose.CoCompose}.
   * @param subject the subject for which to set the property.
   * @param values the current values of the property.
   * @param elementClass the class of the possible value elements.
   */
  public ModelElementsPropertySource(ModelElement subject, List values, Class elementClass)
  {
    Assert.isNotNull(subject);
    Assert.isNotNull(values);
    this.subject = subject;
    this.values = values;
    createPossibleValues(elementClass);
    createValueStrings();
  }

  /**
   * @see org.eclipse.ui.views.properties.IPropertySource#getEditableValue()
   */
  public Object getEditableValue()
  {
    return this;
  }

  /**
   * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyDescriptors()
   */
  public IPropertyDescriptor[] getPropertyDescriptors()
  {
    try
    {
      if (desc == null)
      {
        desc = createDescriptorsArray(createPropertyDescriptors());
      }
    }
    catch (Exception e)
    {
      getLog().report(e);
    }
    return desc;
  }

  /**
   * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyValue(java.lang.Object)
   */
  public Object getPropertyValue(Object id)
  {
    List values = getValues();
    List possvalues = getValueStrings();
    try
    {
      int index = Integer.parseInt((String) id);
      if (index < values.size())
      {
        String fullname = getFullName((ModelElement) values.get(index));
        return new Integer(possvalues.indexOf(fullname));
      }
      else
        return new Integer(possvalues.indexOf("<<none>>"));
    }
    catch (NumberFormatException e)
    {
      getLog().report(e);
    }
    return null;
  }

  /**
   * @see org.eclipse.ui.views.properties.IPropertySource#isPropertySet(java.lang.Object)
   */
  public boolean isPropertySet(Object id)
  {
    return true;
  }

  /**
   * @see org.eclipse.ui.views.properties.IPropertySource#resetPropertyValue(java.lang.Object)
   */
  public void resetPropertyValue(Object id)
  {
    List values = getValues();
    try
    {
      int index = Integer.parseInt((String) id);
      if (index < values.size())
      {
        removeElement((ModelElement) values.get(index));
      }
    }
    catch (NumberFormatException e)
    {
      getLog().report(e);
    }
  }

  /**
   * @see org.eclipse.ui.views.properties.IPropertySource#setPropertyValue(java.lang.Object, java.lang.Object)
   */
  public void setPropertyValue(Object id, Object value)
  {
    List values = getValues();
    List possValues = getPossibleValues();
    try
    {
      int index = Integer.parseInt((String) id);
      int valueIndex = ((Integer) value).intValue();
      if (index >= values.size())
        index = -1;
      else
        removeElement((ModelElement) values.get(index));
      if (valueIndex > 0)
      {
        addElement(index, (ModelElement) possValues.get(valueIndex - 1));
      }
    }
    catch (NumberFormatException e)
    {
      getLog().report(e);
    }
  }
 
  /**
   * @see java.lang.Object#toString()
   */
  public String toString()
  {
    String value = "[";
    List values = getValues();
    for (int index = 0; index < values.size(); index++)
    {
      value += values.get(index);
      if (index < values.size() - 1)
        value += ", ";
    }
    value += "]";
    return value;
  }

  /**
   * Responsible for removing an element from the property. Currently only
   * updates the dependencies on the shared models defined in
   * {@link be.ac.vub.cocompose.CoCompose}. 
   * @param element the element to remove.
   */
  protected void removeElement(ModelElement element)
  {
    Model model = subject.getModel();
    Model emodel = element.getModel();
    if ((model == null) || (emodel == null))
      return;
    if (model.equals(emodel))
      return;
    getSettings().removeSharedModel(emodel);
  }
  
  /**
   * Responsible for adding an element to the property. Currently only
   * updates the dependencies on the shared models defined in
   * {@link be.ac.vub.cocompose.CoCompose}. 
   * @param index the index where to add or -1 to append. 
   * @param element the element to add.
   */
  protected void addElement(int index, ModelElement element)
  {
    Model model = subject.getModel();
    Model emodel = element.getModel();
    if ((model == null) || (emodel == null))
      return;
    if (model.equals(emodel))
      return;
    getSettings().addSharedModel(emodel);
  }
  
  /**
   * Returns the full model-wide unique name of the given element
   * @param element the element to return the name for
   * @return the full element name
   */
  protected String getFullName(ModelElement element)
  {
    String value = element.getName();
    Namespace ns = element.getNamespace();
    if (ns == null)
      return value;
    while (ns.getNamespace() != null)
    {
      value = ns.getName() + "." + value;
      ns = ns.getNamespace();
    }
    value = ns.getName() + "." + value;
    return value;
  }

  /**
   * Gets the settings object.
   * @return the settings object.
   */
  protected CoCompose getSettings()
  {
    return settings;
  }

  /**
   * Gets the logging object.
   * @return the logging object.
   */
  protected Log getLog()
  {
    return settings.getLog();
  }

  /**
   * Gets the possible value strings for this property.
   * @return the possible value strings for this property.
   */
  protected List getValueStrings()
  {
    return valueStrings;
  }

  /**
   * Gets the possible values for this property.
   * @return the possible values for this property.
   */
  protected List getPossibleValues()
  {
    return possValues;
  }

  /**
   * Gets the values for this property.
   * @return the values for this property.
   */
  protected List getValues()
  {
    return values;
  }

  /**
   * Creates the property descriptors list
   * @return the property descriptors list
   */
  protected List createPropertyDescriptors()
  {
    List values = getValues();
    List possValues = getPossibleValues();
    List descs = new Vector();
    String[] possvalues = createStringArray(getValueStrings());
    Iterator vs = values.iterator();
    while (vs.hasNext())
    {
      Object value = vs.next();
      Integer id = new Integer(values.indexOf(value));
      ComboBoxPropertyDescriptor desc =
        new ComboBoxPropertyDescriptor(
          id.toString(),
          id.toString(),
          possvalues);
      desc.setLabelProvider(new IndexedLabelProvider(possValues));
      descs.add(desc);
    }
    Integer id = new Integer(values.size());
    ComboBoxPropertyDescriptor desc =
      new ComboBoxPropertyDescriptor(id.toString(), id.toString(), possvalues);
    desc.setLabelProvider(new IndexedLabelProvider(possValues));
    descs.add(desc);
    return descs;
  }

  /**
   * Creates the property descriptors array from a Collection.
   * @param desc - the descriptors collection.
   * @return the property descriptors array.
   */
  protected IPropertyDescriptor[] createDescriptorsArray(Collection desc)
  {
    IPropertyDescriptor[] idesc = new IPropertyDescriptor[desc.size()];
    int index = 0;
    Iterator descIt = desc.iterator();
    while (descIt.hasNext())
    {
      idesc[index] = (IPropertyDescriptor) descIt.next();
      index++;
    }
    return idesc;
  }

  /**
   * Creates a string array from a Collection.
   * @param coll - the collection.
   * @return the string array.
   */
  protected String[] createStringArray(Collection coll)
  {
    String[] strings = new String[coll.size()];
    int index = 0;
    Iterator collIt = coll.iterator();
    while (collIt.hasNext())
    {
      strings[index] = (String) collIt.next();
      index++;
    }
    return strings;
  }

  /**
   * Creates the available value strings, given the class.
   * @param elementClass the class of the possible values.
   * @param namespace the namespace in which to look.
   */
  private void createPossibleValues(Class elementClass, Namespace namespace)
  {
    Assert.isNotNull(namespace);
    if (elementClass != null)
    {
      possValues = namespace.getDeepOwnedElements(elementClass);
    }
    else
    {
      possValues = namespace.getDeepOwnedElements(ModelElement.class);
    }
    possValues.remove(subject);
  }

  /**
   * Creates the available value strings, given the class. Looks in all
   * shared models.
   * @param elementClass the class of the possible values.
   */
  private void createPossibleValues(Class elementClass)
  {
    Iterator shared = getSettings().getSharedModels().iterator();
    possValues = new Vector();
    while (shared.hasNext())
    {
      Namespace ns = (Namespace) shared.next();
      if (elementClass != null)
      {
        possValues.addAll(ns.getDeepOwnedElements(elementClass));
        if (elementClass.isInstance(ns))
          possValues.add(ns);
      }
      else
      {
        possValues.addAll(ns.getDeepOwnedElements(ModelElement.class));
        if (ns instanceof ModelElement)
          possValues.add(ns);
      }
    }
    // never point at self or the containing model
    possValues.remove(subject);
    Namespace ns = subject.getRootNamespace();
    if (ns != null)
      possValues.remove(ns);
  }

  /**
   * Creates the available value strings, given the possible values.
   */
  private void createValueStrings()
  {
    valueStrings = new Vector();
    valueStrings.add("<<none>>");
    Iterator elts = possValues.iterator();
    while (elts.hasNext())
    {
      valueStrings.add(getFullName((ModelElement) elts.next()));
    }
  }
}
