/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2003, 2004
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 * 
 * $Id: DefaultNamespace.java,v 1.8 2004/05/18 11:40:13 dwagelaa Exp $
 * $Log: DefaultNamespace.java,v $
 * Revision 1.8  2004/05/18 11:40:13  dwagelaa
 * 0 worked on EPS and SVG export: both work now, except that images aren't supported in SVG
 * 0 worked on ApplySolutionPattern: the result is now properly displayed in the editor, and undoable ;-)
 *
 * Revision 1.7  2004/05/10 14:26:23  dwagelaa
 * worked on shared models and eclipse property sources
 *
 * Revision 1.6  2004/05/07 14:19:27  dwagelaa
 * + added model pool (shared model) support
 * + added inter-model reference support
 *
 * Revision 1.5  2004/04/28 16:30:08  dwagelaa
 * rearranged EPS export under ExportAction
 *
 * Revision 1.4  2004/04/27 16:43:58  dwagelaa
 * + added drag-n-drop
 * + added copy/paste
 * + added general copying operation
 * - removed examples (in separate module now)
 *
 */

package be.ac.vub.cocompose.lang.impl.core;

import java.util.Enumeration;
import java.util.List;
import java.util.Map;
import java.util.Vector;
import java.util.Hashtable;
import be.ac.vub.cocompose.lang.ModelElementException;
import be.ac.vub.cocompose.lang.ModelVisitor;
import be.ac.vub.cocompose.lang.Properties;
import be.ac.vub.cocompose.lang.core.Namespace;
import be.ac.vub.cocompose.lang.core.ModelElement;
import be.ac.vub.cocompose.log.Log;

/**
 * Default container and namespace mechanism for all CoCompose language elements.
 * @version 0.2.11, 17/05/2004
 * @since 0.2.0
 * @author Dennis Wagelaar 
 */
abstract public class DefaultNamespace extends DefaultModelElement implements Namespace
{
  private Vector ownedElements = new Vector();
  private Hashtable ownedNames = new Hashtable();
  private Hashtable ownedIds = new Hashtable();

  /**
   * Accepts a visitor.
   * @param modelVisitor The visitor to accept.
   */
  public void accept(ModelVisitor modelVisitor) throws ModelElementException 
  {
    modelVisitor.visitNamespace(this);
  }

  /**
   * Adds an element to this namespace.
   * @param modelElement - the element to add.
   * @exception ModelElementException - if the name of the element is already used within this namespace. 
   */
  public void addOwnedElement(ModelElement modelElement) throws ModelElementException
  {
    addOwnedElement(-1, modelElement);
  }

  /**
   * Adds an element to this namespace.
   * @param index - the index at which to add the element.
   * @param modelElement - the element to add.
   * @exception ModelElementException - if the name of the element is already used within this namespace. 
   */
  public void addOwnedElement(int index, ModelElement modelElement) throws ModelElementException
  {
    if (modelElement.getName() == null)
    {
      int serial = ownedElements.size();
      while (isRegistered("ModelElement" + serial))
        serial++;
      modelElement.setName("ModelElement" + serial);
    }
    register(modelElement);
    if (index > -1)
      ownedElements.add(index, modelElement);
    else
      ownedElements.add(modelElement);
    modelElement.setNamespace(this);
    fireStructureChange(Properties.ID_OWNED_ELEMENTS, modelElement);
  }

  /**
   * Removes a element from this namespace.
   * @param modelElement - the element to remove. 
   */
  public void removeOwnedElement(ModelElement modelElement)
  {
    ownedElements.remove(modelElement);
    unregister(modelElement);
    fireStructureChange(Properties.ID_OWNED_ELEMENTS, modelElement);
  }

  /**
   * Returns the elements within this namespace.
   * @return a copy of the List of owned elements.
   */
  public List getOwnedElements()
  {
    return (List) ownedElements;
  }

  /**
   * Returns the elements of a given class within this namespace.
   * @param elementClass the element class.
   * @return the List of owned elements.
   */
  public List getOwnedElements(Class elementClass)
  {
    Vector filteredElements = new Vector();
    Enumeration elements = ownedElements.elements();
    while (elements.hasMoreElements())
    {
      Object element = elements.nextElement();
      if (elementClass.isInstance(element))
        filteredElements.add(element);
    }
    return filteredElements;
  }

  /**
   * Returns the elements of a given class within this namespace and nested namespaces.
   * @param elementClass the element class.
   * @return the List of owned elements.
   */
  public List getDeepOwnedElements(Class elementClass)
  {
    Vector filteredElements = new Vector();
    Enumeration elements = ownedElements.elements();
    while (elements.hasMoreElements())
    {
      Object element = elements.nextElement();
      if (elementClass.isInstance(element))
        filteredElements.add(element);
      if (element instanceof Namespace)
        filteredElements.addAll(((Namespace) element).getDeepOwnedElements(elementClass));
    }
    return filteredElements;
  }

  /**
   * Retrieves an owned element by its name.
   * @param name the name of the element to retrieve.
   * @return the element with the given name or null if it doesn't exist. 
   */
  public ModelElement getOwnedElement(String name)
  {
    ModelElement ownedElement = (ModelElement) ownedNames.get(name);
    return ownedElement;
  }

  /**
   * Retrieves an owned element in this namespace or a nested namespace.
   * @param copy - the (remote) copy of the element to retrieve.
   * @return the owned element or null if it doesn't exist. 
   */
  public ModelElement getDeepOwnedElement(ModelElement copy)
  {
    Namespace ns = copy.getNamespace();
    if (ns == null)
      return null;
    if (ns.getNamespace() != null)
    {
      ModelElement element = getDeepOwnedElement(ns);
      if (element instanceof Namespace)
        return ((Namespace) element).getOwnedElement(copy.getName());
      else
        return null;
    }
    return getOwnedElement(copy.getName());      
  }

  /**
   * Checks whether a name is already registered within this namespace.
   * @param name - the name to check.
   * @return true if and only if the name is already registered. 
   */
  public boolean isRegistered(String name)
  {
    return ownedNames.containsKey(name);
  }

  /**
   * Registers an element name within this namespace.
   * @param oldName - the name to change.
   * @param newName - the name to change into.
   * @exception ModelElementException - if the new name is already used or if the old name does not exist in this namespace.
   */
  public void registerNameChange(String oldName, String newName) throws ModelElementException
  {
    if (ownedNames.containsKey(newName))
      throw new ModelElementException("Name " + newName +
        " not unique within namespace " + getName(), this,
        Log.ERROR);
    if (!ownedNames.containsKey(oldName))
      throw new ModelElementException("Name " + oldName +
        " does not exist within namespace " + getName(), this,
        Log.ERROR);
    Object element = ownedNames.get(oldName);
    ownedNames.remove(oldName);
    ownedNames.put(newName, element);
  }

  /**
   * Returns the global ids for the root namespace. Each id points at its
   * corresponding ModelElement.
   * @return the global ids for the root namespace.
   */  
  public Map getOwnedIds()
  {
    if (getNamespace() != null)
      return getNamespace().getOwnedIds();
    return ownedIds;
  }

  /**
   * Gets the root namespace for this element, if any.
   * @return the root namespace, if any.
   */
  public Namespace getRootNamespace()
  {
    Namespace ns = this;
    while (ns.getNamespace() != null)
    {
      ns = ns.getNamespace();
    }
    return ns;
  }

  /**
   * Translates the given ownedElements index to an otherList index.
   * @param index - the index to translate.
   * @param otherList - the other list to which the index should be translated.
   * @return the translated index.
   */
  protected int translateIndex(int index, List otherList)
  {
    return translateIndex(index, ownedElements, otherList);
  }

  /**
   * Translates the given ownedElements index to an otherList index.
   * @param index - the index to translate.
   * @param sourceList - the list to which the index refers.
   * @param otherList - the other list to which the index should be translated.
   * @return the translated index.
   */
  protected int translateIndex(int index, List sourceList, List otherList)
  {
    if (index < 1)
      return index; 
    Object element;
    do
    {
      index--;
      element = sourceList.get(index);
    }
    while (!(otherList.contains(element)) && (index > 0));
    if (index > 0)
    {
      index = otherList.indexOf(element) + 1;
    }
    return index;
  }

  /**
   * Returns a new Id for the given ModelElement.
   * @param modelElement - the element to get the Id for.
   * @return The element Id or null. 
   */
  private String getOwnedElementId(ModelElement modelElement)
  {
    Map ownedIds = getOwnedIds();
    int subid = 0;
    while (ownedIds.containsKey(Integer.toString(subid)))
    {
      subid++;
    }
    return Integer.toString(subid);
  }

  /**
   * Registers an element within this namespace.
   * @param element the element to register.
   * @exception ModelElementException if the name is already used in this namespace.
   */
  private void register(ModelElement element) throws ModelElementException
  {
    checkId(element, getOwnedIds());
    if (ownedNames.containsKey(element.getName()))
    {
      throw new ModelElementException("Name " + element.getName() +
        " not unique within namespace " + getName(), element,
        Log.ERROR);
    }
    ownedNames.put(element.getName(), element);
  }

  /**
   * Checks the id of the given model element and adds it to the given ownedIds
   * map if it is unique. 
   * @param element the element to check.
   * @param ownedIds the ownedIds map to check against.
   * @throws ModelElementException if an id was not unique.
   */
  private void checkId(ModelElement element, Map ownedIds)
    throws ModelElementException
  {
    if (element.getId() != null)
    {
      if (ownedIds.containsKey(element.getId()))
      {
        throw new ModelElementException("Id " + element.getId() +
          " not unique", element, Log.ERROR);
      }
    }
    else
    {
      element.setId(getOwnedElementId(element));
    }
    ownedIds.put(element.getId(), element);
    if (element instanceof Namespace)
    { // check ids of children
      Namespace ns = (Namespace) element;
      List ownedElts = ns.getOwnedElements();
      for (int index = 0; index < ownedElts.size(); index++)
      {
        checkId((ModelElement) ownedElts.get(index), ownedIds);
      }
    }
  }

  /**
   * Unregisters an element from this namespace.
   * @param element the element to unregister.
   */
  private void unregister(ModelElement element)
  {
    getOwnedIds().remove(element.getId());
    ownedNames.remove(element.getName());
  }
}
