/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2004
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 * 
 * $Id: CopyElement.java,v 1.4 2004/05/18 11:40:13 dwagelaa Exp $
 * $Log: CopyElement.java,v $
 * Revision 1.4  2004/05/18 11:40:13  dwagelaa
 * 0 worked on EPS and SVG export: both work now, except that images aren't supported in SVG
 * 0 worked on ApplySolutionPattern: the result is now properly displayed in the editor, and undoable ;-)
 *
 * Revision 1.3  2004/05/17 14:02:54  dwagelaa
 * + added initial ApplySolutionPattern functionality
 * 0 changed EPSGraphics to AWTGraphics
 * + added initial SVG export functionality (EPS works by now)
 * 0 updated version numbers to 0.2.11
 *
 * Revision 1.2  2004/04/28 12:24:36  dwagelaa
 * worked on drag-n-drop
 * updated copy/paste
 * updated deep copy operation
 *
 * Revision 1.1  2004/04/27 16:43:58  dwagelaa
 * + added drag-n-drop
 * + added copy/paste
 * + added general copying operation
 * - removed examples (in separate module now)
 *
 */

package be.ac.vub.cocompose.operations;

import java.awt.Point;
import java.util.Hashtable;
import java.util.List;
import java.util.Vector;

import be.ac.vub.cocompose.lang.DefaultSuperModelVisitor;
import be.ac.vub.cocompose.lang.ModelElementException;
import be.ac.vub.cocompose.lang.ModelFactory;
import be.ac.vub.cocompose.lang.VisitedElement;
import be.ac.vub.cocompose.lang.constraints.AndConstraint;
import be.ac.vub.cocompose.lang.constraints.ElementConstraint;
import be.ac.vub.cocompose.lang.constraints.OrConstraint;
import be.ac.vub.cocompose.lang.core.Concept;
import be.ac.vub.cocompose.lang.core.ImplementationGenerator;
import be.ac.vub.cocompose.lang.core.ImplementationPattern;
import be.ac.vub.cocompose.lang.core.ImplementationRolePart;
import be.ac.vub.cocompose.lang.core.Model;
import be.ac.vub.cocompose.lang.core.ModelElement;
import be.ac.vub.cocompose.lang.core.ModelTransformation;
import be.ac.vub.cocompose.lang.core.Namespace;
import be.ac.vub.cocompose.lang.core.RefinedElement;
import be.ac.vub.cocompose.lang.core.RelationElement;
import be.ac.vub.cocompose.lang.core.Relationship;
import be.ac.vub.cocompose.lang.core.Role;
import be.ac.vub.cocompose.lang.core.SolutionPattern;
import be.ac.vub.cocompose.lang.datatypes.Multiplicity;
import be.ac.vub.cocompose.lang.properties.ModelProperty;
import be.ac.vub.cocompose.lang.properties.SimpleProperty;

/**
 * Copies a CoCompose element. Uses {@link CopyElementLinks} to copy the
 * non-containment links in a second pass.
 * @author Dennis Wagelaar
 * @version 0.2.11, 17/05/2004
 * @since 0.2.9
 * @see be.ac.vub.cocompose.operations.CopyElementLinks
 */
public class CopyElement extends DefaultSuperModelVisitor
{
  private ModelFactory factory = null;
  private VisitedElement copy = null;
  /**
   * Mapping of original elements to their copies: 
   * <pre>Map&lt;VisitedElement |-&gt; VisitedElement&gt;</pre>
   */
  protected Hashtable elementMap = new Hashtable();
  protected CopyElementLinks copyLinks = new CopyElementLinks(elementMap);
  
  /**
   * Creates a CopyElement operation.
   * @param factory the factory to create new elements.
   */
  public CopyElement(ModelFactory factory)
  {
    if (factory == null)
      throw new Error("factory may not be null");
    this.factory = factory;
  }

  /**
   * Deep copies the given element and returns it.
   * @param element the element to copy.
   * @return the copied element.
   * @throws ModelElementException
   */
  public VisitedElement copy(VisitedElement element) throws ModelElementException
  {
    visit(element);
    copyLinks.setCopy(copy);
    copyLinks.visit(element);
    return copy;
  }
  
  /**
   * Deep copied the given list of elements.
   * @param elements the elements to copy.
   * @return the copied elements.
   * @throws ModelElementException
   */
  public List copy(List elements) throws ModelElementException
  {
    List copies = new Vector();
    for (int i = 0; i < elements.size(); i ++)
    {
      copies.add(copy((VisitedElement) elements.get(i)));
    }
    return copies;
  }

  /**
   * Visits a ModelElement.
   * @param modelElement - the element to visit.
   * @exception ModelElementException
   */
  public void visitModelElement(ModelElement modelElement) throws ModelElementException
  {
    super.visitModelElement(modelElement);
    ModelElement cp = (ModelElement) copy;
    elementMap.put(modelElement, copy);
    cp.setId(modelElement.getId());
    cp.setName(modelElement.getName());
  }

  /**
   * Visits a Namespace.
   * @param namespace - the element to visit.
   * @exception ModelElementException
   */
  public void visitNamespace(Namespace namespace) throws ModelElementException
  {
    super.visitNamespace(namespace);
    Namespace ns = (Namespace) copy;
    CopyElement nestedcopy = new CopyElement(factory);
    List owned = namespace.getOwnedElements();
    for (int i = 0; i < owned.size(); i ++)
    {
      visit((ModelElement) owned.get(i));
      ModelElement localel = (ModelElement) getCopy();
      ns.addOwnedElement(localel);
      localel.setNamespace(ns);
    }
    copy = ns; // re-set copy to the current element
  }

  /**
   * Visits a Model.
   * @param model - the element to visit.
   * @exception ModelElementException
   */
  public void visitModel(Model model) throws ModelElementException
  {
    copy = factory.createModel();
    super.visitModel(model);
    Model m = (Model) copy;
    m.setUri(model.getUri());
  }

  /**
   * Visits a Relationship.
   * @param relationship - the element to visit.
   * @exception ModelElementException
   */
  public void visitRelationship(Relationship relationship) throws ModelElementException
  {
    copy = factory.createRelationship();
    super.visitRelationship(relationship);
  }

  /**
   * Visits a SolutionPattern.
   * @param solutionPattern - the element to visit.
   * @exception ModelElementException
   */
  public void visitSolutionPattern(SolutionPattern solutionPattern) throws ModelElementException
  {
    copy = factory.createSolutionPattern();
    super.visitSolutionPattern(solutionPattern);
  }

  /**
   * Visits an ImplementationGenerator.
   * @param implementationGenerator - the element to visit.
   * @exception ModelElementException
   */
  public void visitImplementationGenerator(ImplementationGenerator implementationGenerator) throws ModelElementException
  {
    copy = factory.createImplementationGenerator();
    super.visitImplementationGenerator(implementationGenerator);
  }

  /**
   * Visits an ImplementationRolePart.
   * @param rolePart - the element to visit.
   * @exception ModelElementException
   */
  public void visitImplementationRolePart(ImplementationRolePart rolePart) throws ModelElementException
  {
    copy = factory.createImplementationRolePart();
    super.visitImplementationRolePart(rolePart);
  }

  /**
   * Visits an ImplementationPattern.
   * @param implementationPattern - the element to visit.
   * @exception ModelElementException
   */
  public void visitImplementationPattern(ImplementationPattern implementationPattern) throws ModelElementException
  {
    copy = factory.createImplementationPattern();
    super.visitImplementationPattern(implementationPattern);
  }

  /**
   * Visits a ModelTransformation.
   * @param modelTransformation the element to visit.
   * @exception ModelElementException
   */
  public void visitModelTransformation(ModelTransformation modelTransformation) throws ModelElementException
  {
    copy = factory.createModelTransformation();
    super.visitModelTransformation(modelTransformation);
  }

  /**
   * Visits a RelationElement.
   * @param relationElement - the element to visit.
   * @exception ModelElementException
   */
  public void visitRelationElement(RelationElement relationElement) throws ModelElementException
  {
    super.visitRelationElement(relationElement);
    RelationElement relel = (RelationElement) copy;
    relel.setPosition((Point) relationElement.getPosition().clone());
    relel.setBackgroundColor(relationElement.getBackgroundColor());
  }

  /**
   * Visits a RefinedElement.
   * @param refinedElement - the element to visit.
   * @exception ModelElementException
   */
  public void visitRefinedElement(RefinedElement refinedElement) throws ModelElementException
  {
    super.visitRefinedElement(refinedElement);
    RefinedElement refel = (RefinedElement) copy;
    refel.setIconUri(refinedElement.getIconUri());
  }

  /**
   * Visits a concept.
   * @param concept - the element to visit.
   * @exception ModelElementException
   */
  public void visitConcept(Concept concept) throws ModelElementException
  {
    copy = factory.createConcept();
    super.visitConcept(concept);
  }

  /**
   * Visits a Role.
   * @param role - the element to visit.
   * @exception ModelElementException
   */
  public void visitRole(Role role) throws ModelElementException
  {
    copy = factory.createRole();
    super.visitRole(role);
    Role r = (Role) copy;
    visit(role.getMultiplicity());
    r.setMultiplicity((Multiplicity) getCopy());
    copy = r; //reset copy to original element
  }

  /**
   * Visits an ElementConstraint.
   * @param elementConstraint - the element to visit.
   * @exception ModelElementException
   */
  public void visitElementConstraint(ElementConstraint elementConstraint) throws ModelElementException
  {
    copy = factory.createElementConstraint();
    super.visitElementConstraint(elementConstraint);
  }

  /**
   * Visits an AndConstraint.
   * @param andConstraint - the element to visit.
   * @exception ModelElementException
   */
  public void visitAndConstraint(AndConstraint andConstraint) throws ModelElementException
  {
    copy = factory.createAndConstraint();
    super.visitAndConstraint(andConstraint);
  }

  /**
   * Visits an OrConstraint.
   * @param orConstraint - the element to visit.
   * @exception ModelElementException
   */
  public void visitOrConstraint(OrConstraint orConstraint) throws ModelElementException
  {
    copy = factory.createOrConstraint();
    super.visitOrConstraint(orConstraint);
  }

  /**
   * Visits a SimpleProperty.
   * @param simpleProperty - the element to visit.
   * @exception ModelElementException
   */
  public void visitSimpleProperty(SimpleProperty simpleProperty) throws ModelElementException
  {
    copy = factory.createSimpleProperty();
    super.visitSimpleProperty(simpleProperty);
  }

  /**
   * Visits a ModelProperty.
   * @param modelProperty - the element to visit.
   * @exception ModelElementException
   */
  public void visitModelProperty(ModelProperty modelProperty) throws ModelElementException
  {
    copy = factory.createModelProperty();
    super.visitModelProperty(modelProperty);
  }

  /**
   * Visits a Multiplicity.
   * @param multiplicity - the element to visit.
   * @exception ModelElementException
   */
  public void visitMultiplicity(Multiplicity multiplicity) throws ModelElementException
  {
    copy = new Multiplicity(multiplicity);
  }
  
  /**
   * Gets the generated copy.
   * @return the generated copy.
   */
  public VisitedElement getCopy()
  {
    return copy;
  }
  
  /**
   * Resets this copier for a new copy operation.
   */
  public void reset()
  {
    copy = null;
    elementMap.clear();
  }
}
