/*--------------------------------------------------------------------------------
 Copyright Stichting Mathematisch Centrum 2000

  Permission to use and distribute this software and its
  documentation for any purpose is hereby granted without fee, provided that
  the above copyright notice, author statement and this permission,
  warranties and liability notice appear in all copies of this software
  and related documentation.

  Warranties
  Stichting Mathematisch Centrum makes no warranties, express or implied, with
  regard to this software. In particular, but without limitation, SMC makes
  no warranty of fitness for a particular use, no warranty of freedom from
  infringement of any patent or copyright, nor any warranty as to accuracy.
  Accordingly, SMC assumes no obligation to furnish assistance of any kind
  whatsoever or to furnish additional information or documentation.

  Liability
  In no event shall Stichting Mathematisch Centrum be liable for any damages
  arising out of or in connection with the use or performance of this
  software.
*/
package cwi.SVGGraphics;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.awt.image.*;
import java.awt.print.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import java.net.URL;


/**
* Test program for {@link SVGGraphics SVGGraphics}. Users might want to look at this file
* as an example of generating SVG.
* <p>
* In fact, this class is just a wrapper around {@link STestCanvas STestCanvas} where the real
* drawing is done.
*
* @author Ivan Herman
*/
public class STest extends JFrame {
    static RenderingHints renderingHintsHigh;
  
    static {
        renderingHintsHigh = new RenderingHints(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_ON);
        renderingHintsHigh.put(RenderingHints.KEY_COLOR_RENDERING,RenderingHints.VALUE_COLOR_RENDER_QUALITY);
        renderingHintsHigh.put(RenderingHints.KEY_ALPHA_INTERPOLATION,RenderingHints.VALUE_ALPHA_INTERPOLATION_SPEED);
        renderingHintsHigh.put(RenderingHints.KEY_TEXT_ANTIALIASING,RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
    }

    STestCanvas canvas;

    public class act1 extends AbstractAction
    {
        public act1(String name)
        {
            super(name);
        }
        public act1()
        {
            super();
        }
        
        public void actionPerformed(ActionEvent e)
        {
            if( e.getActionCommand().equals("Quit") ) {
                System.exit(0);
            }
        }
    }
    
    public class svg extends AbstractAction
    {        
        public svg(String name)
        {
            super(name);
        }
        public svg()
        {
            super();
        }
        
        public void actionPerformed(ActionEvent e)
        {
          try {
            SVGGraphics svg = SVGGraphics.create(canvas);
            if( svg == null ) return;
            canvas.draw(svg,canvas.getBounds(),Color.white,Color.black);
            svg.close();
            System.out.println("SVG File generated");
          } catch( Exception ee ) {
            System.out.println(ee.toString());
          }
        }
    }

    public STest()
    {
        super("Swing Test");

        JMenuBar menus  = new JMenuBar();
        JMenu    menu   = new JMenu("File");

        Action qqqq = new svg();
        qqqq.putValue(Action.NAME,"Generate SVG");
        JMenuItem rr = menu.add(qqqq);

        Action actq = new act1();
        actq.putValue(Action.NAME,"Quit");
        JMenuItem second = menu.add(actq);
        menus.add(menu);

        this.setJMenuBar(menus);

        canvas = new STestCanvas();
		    canvas.setBackground(Color.white);
        this.getContentPane().add(canvas,"Center");
        this.pack();
        this.setSize(600,600);
        this.setLocation(200,200);
        this.setVisible(true);
        this.show();
        this.repaint();
        this.addKeyListener(new KeyAdapter() {
            public void KeyPressed(KeyEvent e) {
                System.out.println("%%%% " + e.getKeyCode());
            }
        });
    }
    
    public static void main(String[] args)
    {
	String systemUI = UIManager.getSystemLookAndFeelClassName();

        try {
            UIManager.setLookAndFeel(systemUI);
        } catch( Exception exc ) {;}

        STest cols = new STest();
    }
}

/**
* Simple test canvas to do some simple Java2D drawing. Used to test the SVG generation.
*
* @author Ivan Herman
*/
class STestCanvas extends JPanel
{
    BufferedImage image = null;
    Image testImage     = null;
    private boolean errorflag = false;
        
    public Image getImage()
    {
        ImageObserver observer = new ImageObserver() {
            public boolean imageUpdate(Image img,
                                       int infoflags,
                                       int x,
                                       int y,
                                       int width,
                                       int height)
            {
                if( (infoflags & ImageObserver.ABORT) != 0 ||
                    (infoflags & ImageObserver.ERROR) != 0 ) {
                    errorflag = true;
                    return false;
                } else {
                    return true;
                }
            }
        };
        try {
            Image img = Toolkit.getDefaultToolkit().getImage("cwi/SVGGraphics/STest.gif");
            int width = -1;
            int height = -1;
            errorflag = false;
            while( width == -1 ) {
                width = img.getWidth(observer);
                if( errorflag == true ) { return null; }
            }
            errorflag = false;
            while( height == -1 ) {
                height = img.getHeight(observer);
                if( errorflag == true ) { return null; }
            }
            return SVGImageFilter.addURL(img,"/cwi/SVGGraphics/STest.gif");
        } catch( Exception e ) {
            return null;
        }
    }
    
    STestCanvas()
    {
        testImage = getImage();
    }
    
    /**
    * The method wich really does the drawing. The argument <code>g</code> can be either
    * a traditional <code>Graphics2D</code> object, or an instance
    * of {@link SVGGraphics SVGGraphics}.
    */ 
    public void draw(Graphics2D g,Rectangle bounds, Color background, Color foreground)
    {
        if( g instanceof PrinterGraphics || g instanceof SVGGraphics ) {
          ;
        } else {
          g.setRenderingHints(STest.renderingHintsHigh);
          g.setColor(background);
          g.fillRect(0,0,bounds.width,bounds.height);
          g.setColor(foreground);
        }
    
        AffineTransform at = new AffineTransform();
        at.setToScale(((float)bounds.width)/400.0f,((float)bounds.height)/400.0f);

        g.transform(at);

        float dp[] = new float[]{12.0f,5.0f};
        BasicStroke stroke = new BasicStroke(0.5f,
                                             BasicStroke.CAP_SQUARE,
                                             BasicStroke.JOIN_ROUND);
        g.setStroke(stroke);

        Paint paint = new GradientPaint(20.0f,   20.0f,  Color.red,
                                                300.0f,300.0f,Color.blue);
        g.setPaint(paint);
        stroke = new BasicStroke(1.0f,BasicStroke.CAP_SQUARE,
                                 BasicStroke.JOIN_ROUND);
        g.setStroke(stroke);

        GeneralPath path = new GeneralPath();

        Line2D.Float line1 = new Line2D.Float(20.0f,20.0f,100.0f,300.0f);
        Line2D.Float line2 = new Line2D.Float(200.0f,300.0f,30.0f,30.0f);

        path.append(line1,false);
        path.append(line2,true);

        g.draw(path);
        
        // Text testing...
        Font f;
        f = new Font("Dialog",Font.PLAIN,15);
        g.setFont(f);
        g.drawString("Dialog Plain",150f,50f);
        f = new Font("Dialog",Font.BOLD,15);
        g.setFont(f);
        g.drawString("Dialog Bold",150f,65f);

        f = new Font("DialogInput",Font.PLAIN,15);
        g.setFont(f);
        g.drawString("DialogInput",150f,80f);
        
        f = new Font("Serif",Font.PLAIN,15);
        g.setFont(f);
        g.drawString("Serif",280f,50f);
       
        f = new Font("SansSerif",Font.PLAIN,15);
        g.setFont(f);
        g.drawString("SansSerif",280f,65f);
        
        f = new Font("Monospaced",Font.PLAIN,15);
        g.setFont(f);
        g.drawString("Monospaced",280f,80f);
        
        if( testImage == null ) System.out.println("Image pointer is null???");
        else {
          g.drawImage(testImage,280,100,this);
        }
        
        //g.setColor(Color.red);
        paint = new GradientPaint(0.0f,   0.0f,  Color.red,
                                                200.0f,-100.0f,Color.blue);
        g.setPaint(paint);
        path = new GeneralPath(GeneralPath.WIND_EVEN_ODD);
        path.moveTo(0.0f,0.0f);
        path.lineTo(200.0f,0.0f);
        path.lineTo(200.0f,-100.0f);
        path.lineTo(0.0f,-100.0f);
        path.closePath();
        at.setToTranslation(100.0,300.0);
        g.transform(at);

        g.fill(path);

        //====
        g.setColor(Color.blue);
        AlphaComposite comp = AlphaComposite.getInstance(
            AlphaComposite.SRC_OVER,0.5f);
        g.setComposite(comp);
        at.setToRotation(-Math.PI/4.0);
        g.transform(at);
        g.fill(path);
    }

    /**
    * This is just to override the inherited <code>paint</code> operation.
    */
    public void paint(Graphics g1)
    {
        Rectangle bounds = getBounds();
        Graphics2D gDisplay = (Graphics2D) g1;

        if( image == null ) {
            // This is the first run!
            GraphicsConfiguration conf = gDisplay.getDeviceConfiguration();
            image = conf.createCompatibleImage(bounds.width,bounds.height);
            Graphics2D g = (Graphics2D) image.getGraphics();
            draw(g,bounds,gDisplay.getBackground(), gDisplay.getColor());  
        }
        gDisplay.drawImage(image, new AffineTransform(),this);
    }
    
}
    


                
        